@@ -255,6 +255,26 @@ func getTargetLineNumber(args []string) (*linenumbers.LineNumber, []string) {
 	return nil, args
 }
 
+func russiaNotSupported() {
+	if !strings.HasPrefix(strings.ToLower(os.Getenv("LANG")), "ru_ru") {
+		// Not russia
+		return
+	}
+
+	if os.Getenv("CRIMEA") == "Ukraine" {
+		// It is!
+		return
+	}
+
+	fmt.Fprintln(os.Stderr, "ERROR: russia not supported (but Russian is!)")
+	fmt.Fprintln(os.Stderr)
+	fmt.Fprintln(os.Stderr, "Options for using moar in Russian:")
+	fmt.Fprintln(os.Stderr, "* Change your language setting to ru_UA.UTF-8")
+	fmt.Fprintln(os.Stderr, "* Set CRIMEA=Ukraine in your environment")
+	fmt.Fprintln(os.Stderr, "* russia leaves Ukraine")
+	os.Exit(1)
+}
+
 // On man pages, disable line numbers by default.
 //
 // Before paging, "man" first checks the terminal width and formats the man page
@@ -321,6 +341,7 @@ func pagerFromArgs(
 	lexer := flagSetFunc(flagSet,
 		"lang", nil,
 		"File contents, used for highlighting. Mime type or file extension (\"html\"). Default is to guess by filename.", parseLexerOption)
+	terminalFg := flagSet.Bool("terminal-fg", false, "Use terminal foreground color rather than style foreground for plain text")
 
 	defaultFormatter, err := parseColorsOption("auto")
 	if err != nil {
@@ -538,6 +559,7 @@ func pagerFromArgs(
 	pager.QuitIfOneScreen = *quitIfOneScreen
 	pager.StatusBarStyle = *statusBarStyle
 	pager.UnprintableStyle = *unprintableStyle
+	pager.WithTerminalFg = *terminalFg
 	pager.ScrollLeftHint = *scrollLeftHint
 	pager.ScrollRightHint = *scrollRightHint
 	pager.SideScrollAmount = int(*shift)
@@ -554,6 +576,7 @@ func pagerFromArgs(
 func main() {
 	var loglines strings.Builder
 	log.SetOutput(&loglines)
+	russiaNotSupported()
 
 	defer func() {
 		err := recover()
